 // Camadas de base
 var googleStreets = L.tileLayer('http://{s}.google.com/vt?lyrs=m&x={x}&y={y}&z={z}', {
    maxZoom: 22,
    subdomains: ['mt0', 'mt1', 'mt2', 'mt3']
});

var googleHybrid = L.tileLayer('http://{s}.google.com/vt?lyrs=s,h&x={x}&y={y}&z={z}', {
    maxZoom: 22,
    subdomains: ['mt0', 'mt1', 'mt2', 'mt3']
});
// Primeira camada WMS para sisprom:jequie-02
var wmsJequieLayer = L.tileLayer.wms("https://sisprom-br.com.br:8443/geoserver/sisprom/wms", {
    layers: 'sisprom:jequie-02',
    format: 'image/png',
    transparent: true,
    version: '1.1.0',
    srs: 'EPSG:32724',
    minZoom: 10,
    maxZoom: 22
});
// Segunda camada WMS para o contorno de Jequié
var wmsContornoLayer = L.tileLayer.wms("https://sisprom-br.com.br:8443/geoserver/sisprom/wms", {
    layers: 'contorno_jequie',
    format: 'image/png',
    transparent: true,
    version: '1.1.0',
    srs: 'EPSG:32724',
    minZoom: 10,
    maxZoom: 22
});
// Terceira camada WMS para o logradouro de Jequié
var wmsImoveisLayer = L.tileLayer.wms("https://sisprom-br.com.br:8443/geoserver/sisprom/wms", {
    layers: 'sisprom:imoveis_jequie',
    format: 'image/png',
    transparent: true,
    version: '1.1.0',
    srs: 'EPSG:32724',
    minZoom: 10,
    maxZoom: 22
});
// Quarta camada WMS para o logradouro de Jequié
var wmsQuadraLayer = L.tileLayer.wms("https://sisprom-br.com.br:8443/geoserver/sisprom/wms", {
    layers: 'sisprom:QUADRA_PINDORAMA',
    format: 'image/png',
    transparent: true,
    version: '1.1.0',
    srs: 'EPSG:32724',
    minZoom: 10,
    maxZoom: 22
});
// Inicializa o mapa
var mapacadastro = L.map("map", {
    center: [-13.8415, -40.0724],
    zoom: 15,
    minZoom: 10,
    maxZoom: 22,
    zoomControl: false,
    editable: true,
    layers: [googleHybrid]
});

var measureControl = new L.Control.Measure({
    position: 'topleft',
    primaryLengthUnit: 'meters',
    secondaryLengthUnit: 'kilometers',
    primaryAreaUnit: 'sqmeters',
    secondaryAreaUnit: 'hectares'
});
measureControl.addTo(mapacadastro);
document.getElementsByClassName('leaflet-control-measure-toggle')[0].innerHTML = '';
document.getElementsByClassName('leaflet-control-measure-toggle')[0].className += ' fas fa-ruler';
var bounds_group = new L.featureGroup([]);
function setBounds() {
}
setBounds();
// Adiciona controle de zoom
var zoom_bar = new L.Control.ZoomBar({ position: 'topleft' }).addTo(mapacadastro);
// Define as camadas base
var baseLayers = {
    "Google Hybrid": googleHybrid,
    "Google Streets": googleStreets,
    "Ortophoto Jan2024": wmsJequieLayer
};
// Adiciona o controle de camadas ao mapa
L.control.layers(baseLayers).addTo(mapacadastro);

var drawnItems = new L.FeatureGroup().addTo(mapacadastro);
var drawControl = new L.Control.Draw({
    edit: {
        featureGroup: drawnItems, // Grupo de camadas que podem ser editadas
        remove: true // Permite remoção
    },
    draw: {
        polygon: true,
        polyline: false,
        rectangle: false,
        circle: false,
        marker: false,
        circlemarker: false
    }
}).addTo(mapacadastro);
// Carregar o GeoJSON ao iniciar a página
loadGeoJSON();

function loadGeoJSON() {
    fetch('vendor/geojequie/geojson_edit/poligononuvem.geojson')
        .then(response => {
            if (!response.ok) {
                throw new Error(`Erro ao carregar GeoJSON: ${response.statusText}`);
            }
            return response.json();
        })
        .then(data => {
            geojsonData = data; // Salvar globalmente, se necessário
            drawnItems.clearLayers(); // Limpar as camadas existentes
            L.geoJSON(data, {
                style: {
                    color: 'blue',
                    fillColor: 'cyan',
                    fillOpacity: 0.5,
                    weight: 2
                },
                onEachFeature: function (feature, layer) {
                    layer.bindPopup(`  
                            <strong>Nome:</strong> ${feature.properties.nome}<br>
                            <strong>Inscrição:</strong> ${feature.properties.inscricao}<br>
                            <strong>Zoneamento:</strong> ${feature.properties.zoneamento}<br>
                            <strong>Bairro:</strong> ${feature.properties.bairro}<br>
                            <strong>Utilização:</strong> ${feature.properties.utilizacao}<br>
                            <strong>Latitude:</strong> ${layer.getBounds().getCenter().lat.toFixed(6)}<br>
                            <strong>Longitude:</strong> ${layer.getBounds().getCenter().lng.toFixed(6)}
                        `);
                    drawnItems.addLayer(layer);
                }
            });
        })
        .catch(error => {
            console.error("Erro ao carregar GeoJSON:", error);
        });
}

mapacadastro.on('draw:created', function (e) {
    var layer = e.layer;
    layer.setStyle({
        color: 'green', // Cor da borda
        fillColor: 'yellow', // Cor de preenchimento
        fillOpacity: 0.4, // Opacidade do preenchimento
        weight: 3 // Largura da borda
    });
    drawnItems.addLayer(layer);
    var geojson = layer.toGeoJSON();
    openSidebar(geojson, layer);
});

mapacadastro.on('draw:edited', function (e) {
    var layers = e.layers;

    layers.eachLayer(function (layer) {
        // Verifique se o layer tem bounds válidos
        if (layer.getBounds && layer.getBounds().isValid()) {
            var geojson = layer.toGeoJSON();

            if (geojson.geometry && geojson.geometry.coordinates) {
                // Certifique-se de que é um tipo MultiPolygon, mesmo que originalmente seja Polygon
                if (geojson.geometry.type === 'Polygon') {
                    geojson.geometry.type = 'MultiPolygon';
                    geojson.geometry.coordinates = [geojson.geometry.coordinates];
                }
                // Salvar alterações no servidor
                saveGeoJSON(geojson);
                // Opcional: atualize os campos da sidebar
                const lat = layer.getBounds().getCenter().lat.toFixed(6);
                const lng = layer.getBounds().getCenter().lng.toFixed(6);

                document.getElementById('latitude').value = lat;
                document.getElementById('longitude').value = lng;
            } else {
                console.error("Geometria inválida no layer editado.");
            }
        } else {
            console.error("Layer inválido ou sem bounds.");
        }
    });
});

mapacadastro.on('draw:deleted', function (e) {
    e.layers.eachLayer(function (layer) {
        var geojson = layer.toGeoJSON();
        deleteGeoJSON(geojson); // Salva após exclusão do GeoJSON
    });
});

function openSidebar(geojson, layer) {
    document.getElementById('sidebarcadastro').style.display = 'block';
    // Verifique se os bounds e o centro são válidos
    if (layer.getBounds && layer.getBounds().isValid()) {
        const lat = layer.getBounds().getCenter().lat.toFixed(6);
        const lng = layer.getBounds().getCenter().lng.toFixed(6);

        document.getElementById('latitude').value = lat;
        document.getElementById('longitude').value = lng;
    } else {
        console.error("Layer inválido ou sem coordenadas.");
    }

    document.getElementById('nome').value = geojson.properties.nome || '';
    document.getElementById('inscricao').value = geojson.properties.inscricao || '';
    document.getElementById('zoneamento').value = geojson.properties.zoneamento || '';
    document.getElementById('bairro').value = geojson.properties.bairro || '';
    document.getElementById('utilizacao').value = geojson.properties.utilizacao || '';

    document.getElementById('saveBtn').onclick = function () {
        geojson.properties.nome = document.getElementById('nome').value;
        geojson.properties.inscricao = document.getElementById('inscricao').value;
        geojson.properties.zoneamento = document.getElementById('zoneamento').value;
        geojson.properties.bairro = document.getElementById('bairro').value;
        geojson.properties.utilizacao = document.getElementById('utilizacao').value;

        saveGeoJSON(geojson);
        closeSidebar();
    };
}

function closeSidebar(event) {
    // Previne qualquer ação padrão (como recarregar a página)
    event.preventDefault();
    event.stopPropagation(); // Impede que o evento se propague para outros elementos
    // Apenas esconde a sidebar
    document.getElementById('sidebarcadastro').style.display = 'none';
}
// Função para salvar o GeoJSON
function saveGeoJSON(geojson) {
    if (
        !geojson.geometry ||
        !geojson.geometry.coordinates ||
        geojson.geometry.coordinates.length === 0
    ) {
        alert("Geometria inválida. Não foi possível salvar.");
        return;
    }

    fetch('vendor/geojequie/save_geojson.php', {
        method: 'POST',
        headers: { 'Content-Type': 'application/json' },
        body: JSON.stringify(geojson)
    })
        .then(response => response.json())
        .then(data => {
            if (data.success) {
                alert(data.message);
                loadGeoJSON();
            } else {
                alert(`Erro: ${data.message}`);
            }
        })
        .catch(error => {
            console.error("Erro ao salvar o GeoJSON:", error);
        });
}
// Função para deletar o GeoJSON (depois da exclusão)
function deleteGeoJSON(geojson) {
    // Verifique se as coordenadas estão presentes antes de tentar excluir
    if (!geojson.geometry || !geojson.geometry.coordinates) {
        return;  // Retorna se não houver coordenadas
    }

    fetch('vendor/geojequie/delete_geojson.php', {
        method: 'POST',
        headers: {
            'Content-Type': 'application/json'
        },
        body: JSON.stringify(geojson) // Envia o GeoJSON excluído na requisição
    })
        .then(response => response.json())
        .then(data => {
            alert(data.message); // Exibe a mensagem de sucesso ou erro
            loadGeoJSON();  // Recarregar o GeoJSON atualizado no mapa
        })
        .catch(error => {

            alert("Erro ao deletar o arquivo GeoJSON");
        });
}